/***********************************************************
 * $Id$
 *
 * FancyLibs sane wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSANECONTEXT_H_
#define FANCYSANECONTEXT_H_

#include <fancysane.h>
#include <fancyobject.h>
#include <fancylist.h>
#include <fancydrawingimage.h>

namespace Fancy {

class Directory;
class List;
class URL;

namespace Sane {

class SaneCallback;
class SaneOptionDescriptor;

class FANCYSANE_API SaneContext : virtual public Fancy::Object
{
public:
    typedef Fancy::ObjPtr<SaneContext> ptr_t;

    /**
     * @return A list of SaneDeviceInfo object.
     */
    static Fancy::ObjPtr<Fancy::List> enumDevices();

    /**
     * Instantiate a context for the scan with the given device name in the list
     * returned by @c enumDevices().
     */
    static ptr_t newInstance(const std::string &deviceName);

    /**
     * Instantiate a context for the scan with the given index in the list
     * returned by @c enumDevices().
     */
    static ptr_t newInstance(size_t index);

    /**
     * Instantiate a context for reading the given local filesystem directory,
     * which contain files in the form
     *
     * 0001-300dpi-gray.jpg
     * 0002-150dpi-gray.jpg
     * 0003-300dpi-color.jpg
     * 0004-600dpi-color.jpg
     * ...
     *
     * Supported resolutions are 72, 150, 200, 300, 600 and 1200 DPI.
     * Supported mode strings are "Gray" and "Color".
     *
     * @c scanDurationSecs The fake duration to emulate scan progress in seconds.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Directory> &directory, int scanDurationSecs);

    /**
     * Construct a SaneContext from an URL encompassing all newInstance methods
     * mentioned above.
     *
     * URL formats are:
     *
     * sane:///?name=<deviceName>
     * sane:///?index=<deviceIndex>
     * file:///<dir>?duration=<secs>
     *
     * for instantiating a scanner context by device name, device index or
     * a directory with a scan duration.
     * Additional schemes might be added in the future.
     *
     * An additional argument oop=true may be specified together with
     * an optional timeout in seconds and an optional serverPath to start
     * the actual scanning in a child process.
     * When using the directory based fake scanner be aware of the
     * rather special file requirements documented above.
     *
     * Examples for out-of-process URLs are:
     *
     * sane:///?name=<deviceName>&oop=true
     * sane:///?index=<deviceIndex>&oop=true&timeout=30
     * file:///<dir>?duration=<secs>&oop=true&timeout=30&serverPath=/tmp/fancysane-server
     *
     * The default timeout is 60 seconds, the default server path
     * (specifying the fancy-server binary) is
     * /usr/bin/fancysane-server under UNIX and fancysane-server.exe in
     * the same directory as the application's executable under Windows.
     *
     * Sane functions and some errors are documented at http://www.sane-project.org/html/doc012.html
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Fancy::URL> &url);

    /**
     * set a callback for the image data processing.
     */
    virtual void setCallback(const Fancy::ObjPtr<SaneCallback> & callback) = 0;

    /*
     * Start scanning and return the scanned image.
     * This method reports to the configured callback
     *
     * There is no guarantee, that this SANE context is usable after
     * doScanNow() threw an exception, especially out-of-process
     * scan contexts must be reconstructed in this situation.
     */
    virtual Fancy::ObjPtr<Drawing::Image> doScanNow() = 0;

    /**
     * Request an interruption of a long-running call to
     * doScanNow(). This method should be called from a
     * concurrent thread.
     *
     * doScanNow() will throw an exception when this method is called
     * from a concurrent thread.
     *
     * There is no guarantee, that this SANE context is usable after
     * doScanNow() threw an exception, especially out-of-process
     * scan contexts must be reconstructed in this situation.
     */
    virtual void requestInterruption() = 0;

    /**
     * Return a list of SaneOptionDescriptor objects describing the
     * options of the referenced scanner.
     */
    virtual Fancy::ObjPtr<Fancy::List> getOptionDescriptors() = 0;

    /**
     * Find an option descriptor by name.
     * Standard names are "mode" with values "color" or "gray"
     * and "resolution" with integer DPI values.
     */
    virtual Fancy::ObjPtr<SaneOptionDescriptor> getOptionDescriptor(const std::string &name) = 0;

    /**
      * Return the current option descriptor for the given ID.
      *
      * @param id the id of the option to set as returned by @c SaneOptionDescriptor::getId()
      */
    virtual Fancy::ObjPtr<SaneOptionDescriptor> getOptionDescriptor(int id) = 0;

    /**
     * Return the current option value, which is either a
     * Fancy::String or a Fancy::Boolean or a Fancy::Number instance based on
     * the type of the option.
     *
     * @param id the id of the option to set as returned by @c SaneOptionDescriptor::getId()
     */
    virtual Fancy::ObjPtr<Fancy::Object> getOptionValue(int id) = 0;

    /**
     * @param id the id of the option to set as returned by @c SaneOptionDescriptor::getId()
     * @param value the value of the option, which is either a
     *              Fancy::String or a Fancy::Boolean or a Fancy::Number instance based on
     *              the type of the option. If the passed instance does not match the type of
     *              the option, an exception is thrown.
     */
    virtual void setOptionValue(int id, const Fancy::ObjPtr<Fancy::Object> &value) = 0;

protected:
    SaneContext();
    virtual ~SaneContext();
};

}}// end namespaces

#endif
